/*
	Sintetizador de percusin analgica

	(c) Avelino Herrera Morales

	This software is free software; you can redistribute it and/or modify
	it under the terms of the GNU  General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
																																																																																																							 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU  General Public License for more details.
																																																																																																							 
	You should have received a copy of the GNU  General Public
	License along with this software; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>


/* frecuencia de muestreo de salida */
#define  SAMPLE_RATE           44100
/* frecuencia musical ms baja (para las tablas de ondas) */
#define  LOWEST_MUSIC_FREQ     20
/* nmero de muestras que debe tener la tabla de ondas para cada onda */
/* realizamos este clculo para evitar que se produzcan los efectos de escaln
   en la salida de los osciladores. Con este tamao de tabla la variable de
   incremento para recorrerla ser siempre >=1 */
#define  WAVETABLE_SIZE		   (SAMPLE_RATE / LOWEST_MUSIC_FREQ)


/*  Devuelve el nmero de muestras que hay en m milisegundos. */
#define  GET_SAMPLES_MS(m)  ((m * SAMPLE_RATE) / 10000)
/*  Devuelve el nmero de muestras que hay en m centsimas de segundo. */
#define  GET_SAMPLES_CS(m)  ((m * SAMPLE_RATE) / 100)

/*  Tipos de ondas que puede generar un oscilador. */
#define  WAVETYPE_SIN       1
#define  WAVETYPE_SAWTOOTH  2
#define  WAVETYPE_SQUARE    3
#define  WAVETYPE_NOISE     4

/*  Tipos de filtro disponibles. */
#define  FILTERTYPE_LOWPASS   1
#define  FILTERTYPE_BANDPASS  2
#define  FILTERTYPE_HIGHPASS  3
#define  FILTERTYPE_NOTCH     4

/* nmero de muestras por cada canal que se envan simultneamente */
#define  BUFFER_SIZE  256


typedef struct {
	int init_freq;
	int final_freq;
	float exp_scale;
	float distort;
	int amp_attack_time;
	int amp_decay_time;
	int tick_decay_time;
	float tick_level;
	int t;
	float t_freq;
	} bass_drum;

typedef struct {
	int init_freq;
	int final_freq;
	float exp_scale;
	float detune;
	int filter_type;
	float filter_cutoff;
	float filter_resonance;
	float low, mid, hig;
	int amp_decay_time_noise;
	int amp_decay_time_sine;
	float balance;
	int t;
	float t_freq1;
	float t_freq2;
	} snare_drum;

typedef struct {
	int freq1, freq2, freq3, freq4;
	float var1, var2, var3, var4;
	float cycle1, cycle2, cycle3, cycle4;
	int filter_type;
	float filter_cutoff;
	float filter_resonance;
	float low, mid, hig;
	int amp_decay_time;
	float balance;
	int accent_t;
	int t;
	int t1, t2, t3, t4;
	} hihat;

typedef struct {
	int amp_attack_time;
	int amp_decay_time;
	int num_triggs;
	int delay_time;
	float variation;
	int filter_type;
	float filter_cutoff;
	float filter_resonance;
	float low, mid, hig;
	int t;
	int trig;
	int curr_delay;
	float curr_clap_volume;
	} clap;


/* tablas de ondas */
float sin_wave[WAVETABLE_SIZE];
float sawtooth_wave[WAVETABLE_SIZE];

/* defines para el secuenciador */
/* negras por minuto */
#define  BPM              120
/* nmero de muestras por cada nota del secuenciador (cada nota representa una
   semicorchea en un ritmo de 4/4 */
#define  SAMPLES_PER_ROW  ((SAMPLE_RATE * 60) / (BPM * 4))
/* nmero de semicorcheas en una secuencia de 4 negras */
#define  SEQUENCE_SIZE    16

typedef struct {
	signed char trig;
	float volume;
	} row;

/* arrays donde se almacenan las secuencias de cada instrumento de percusin */
row bass_drum_sequence[SEQUENCE_SIZE];
row snare_drum_sequence[SEQUENCE_SIZE];
row hihat_sequence[SEQUENCE_SIZE];
row cymbal_sequence[SEQUENCE_SIZE];
row clap_sequence[SEQUENCE_SIZE];
/* volmenes actuales para cada instrumento */
float bass_drum_volume;
float snare_drum_volume;
float hihat_volume;
float cymbal_volume;
float clap_volume;



/******************************************************************************
	Inicializa la tabla de ondas.
*/
void init_wavetable(void) {
	int i;

	srand((unsigned int) time(NULL));
	for (i = 0; i < WAVETABLE_SIZE; i++) {
		sin_wave[i] = sin(i * 2 * M_PI / WAVETABLE_SIZE);
		sawtooth_wave[i] = (i * 2.0 / WAVETABLE_SIZE) - 1;
		}
	return;
	}

/******************************************************************************
                                   HI HAT
******************************************************************************/
void hihat_init(hihat *hh, int freq1, int freq2, int freq3, int freq4,
                float var1, float var2, float var3, float var4,
                float cycle1, float cycle2, float cycle3, float cycle4,
                int filter, float cutoff, float reso, int amp_decay,
                float balance, int accent_t) {
	hh->freq1 = freq1;
	hh->freq2 = freq2;
	hh->freq3 = freq3;
	hh->freq4 = freq4;
	hh->var1 = var1;
	hh->var2 = var2;
	hh->var3 = var3;
	hh->var4 = var4;
	hh->cycle1 = cycle1;
	hh->cycle2 = cycle2;
	hh->cycle3 = cycle3;
	hh->cycle4 = cycle4;
	hh->filter_type = filter;
	hh->filter_cutoff = cutoff;
	hh->filter_resonance = reso;
	hh->amp_decay_time = amp_decay;
	hh->balance = balance;
	hh->t1 = hh->t2 = hh->t3 = hh->t4 = -1;
	hh->hig = hh->mid = hh->low = 0.0;
	hh->accent_t = accent_t;
	hh->t = -1;
	return;
	}

void hihat_start(hihat *hh) {
	hh->t = 0.0;
	hh->t1 = hh->t2 = hh->t3 = hh->t4 = 0.0;
	hh->hig = hh->mid = hh->low = 0.0;
	return;
	}

void hihat_stop(hihat *hh) {
	hh->t = -1;
	hh->t1 = hh->t2 = hh->t3 = hh->t4 = -1;
	hh->hig = hh->mid = hh->low = 0.0;
	return;
	}

void hihat_work(hihat *hh, float *output) {
	int i;
	int adecay;
	float amp, noise_amp, colour_amp;
	float inc1, inc2, inc3, inc4;
	float v1, v2, v3, v4;
	float hig, mid, low;
	float noise;
	float var1, var2, var3, var4;

	if (hh->t < 0) {
		memset(output, 0, BUFFER_SIZE * sizeof(float));
		return;
		}
	/* variacin de la frecuencia para cada oscilador */
	var1 = (2.0 * ((float) rand() / RAND_MAX) - 1.0) * hh->var1;
	var2 = (2.0 * ((float) rand() / RAND_MAX) - 1.0) * hh->var2;
	var3 = (2.0 * ((float) rand() / RAND_MAX) - 1.0) * hh->var3;
	var4 = (2.0 * ((float) rand() / RAND_MAX) - 1.0) * hh->var4;
	inc1 = ((float) hh->freq1 + var1) / LOWEST_MUSIC_FREQ;
	inc2 = ((float) hh->freq2 + var2) / LOWEST_MUSIC_FREQ;
	inc3 = ((float) hh->freq3 + var3) / LOWEST_MUSIC_FREQ;
	inc4 = ((float) hh->freq4 + var4) / LOWEST_MUSIC_FREQ;
	/* inicializamos el filtro */
	hig = hh->hig;
	mid = hh->mid;
	low = hh->low;
	/* hallamos los samples para el tiempo de cada */
	adecay = GET_SAMPLES_MS(hh->amp_decay_time);
	for (i = 0; i < BUFFER_SIZE; i++) {
		/* calculamos la envolvente de amplitud */
		amp = 1.0 - ((float) hh->t / adecay);
		if (amp <= 0.0) {
			hh->t = -1;
			break;
			}
		/* obtenemos las salidas de los 4 osciladores de onda cuadrada */
		v1 = (sawtooth_wave[(int) rint(hh->t1)] > hh->cycle1) ? 1 : -1;
		v2 = (sawtooth_wave[(int) rint(hh->t2)] > hh->cycle2) ? 1 : -1;
		v3 = (sawtooth_wave[(int) rint(hh->t3)] > hh->cycle3) ? 1 : -1;
		v4 = (sawtooth_wave[(int) rint(hh->t4)] > hh->cycle4) ? 1 : -1;
		/* y la seal de ruido */
		noise = (2.0 * ((float) rand() / RAND_MAX)) - 1.0;
		/* calculamos la amplitudes del ruido y el color a partir del balance */
		noise_amp = colour_amp = 1.0;
		if (hh->balance > 0)
			colour_amp = 1.0 - hh->balance;
		else if (hh->balance < 0)
			noise_amp = 1.0 + hh->balance;
		/* seal antes de ser filtrada */
		v1 = amp * ((colour_amp * v1 * v2 * v3 * v4) + (noise_amp * noise));
		/* calculamos la resonancia del filtro a partir de acento y filtramos */
		if (hh->t > hh->accent_t)
			hig = v1 - (mid * hh->filter_resonance) - low;
		 else
			hig = v1 - (mid * 0.02) - low;
		mid += hig * hh->filter_cutoff;
		low += mid * hh->filter_cutoff;
		switch (hh->filter_type) {
			case FILTERTYPE_LOWPASS  : output[i] = low; break;
			case FILTERTYPE_HIGHPASS : output[i] = hig; break;
			case FILTERTYPE_BANDPASS : output[i] = mid; break;
			case FILTERTYPE_NOTCH    : output[i] = low + hig; break;
			}
		/* incrementamos las variables que recorren la talba de ondas */
		hh->t1 += inc1;
		if (hh->t1 >= WAVETABLE_SIZE)
			hh->t1 -= WAVETABLE_SIZE;
		hh->t2 += inc2;
		if (hh->t2 >= WAVETABLE_SIZE)
			hh->t2 -= WAVETABLE_SIZE;
		hh->t3 += inc3;
		if (hh->t3 >= WAVETABLE_SIZE)
			hh->t3 -= WAVETABLE_SIZE;
		hh->t4 += inc4;
		if (hh->t4 >= WAVETABLE_SIZE)
			hh->t4 -= WAVETABLE_SIZE;
		/* a por la siguiente muestra */
		hh->t++;
		}
	/* almacenamos el estado del filtro */
	hh->hig = hig;
	hh->mid = mid;
	hh->low = low;
	/* ponemos a 0 lo que queda de buffer */
	for ( ; i < BUFFER_SIZE; i++)
		output[i] = 0;
	return;
	}

/******************************************************************************
                                  SNARE DRUM
******************************************************************************/
void snare_drum_init(snare_drum *sd, int ifreq, int ffreq, float exp_scale,
                     float detune, int filter, float cut, float reso,
                     int amp_decay_n, int amp_decay_s, float balance) {
  	sd->init_freq = ifreq;
	sd->final_freq = ffreq;
	sd->exp_scale = exp_scale;
	sd->detune = detune;
	sd->filter_type = filter;
	sd->filter_cutoff = cut;
	sd->filter_resonance = reso;
	sd->amp_decay_time_noise = amp_decay_n;
	sd->amp_decay_time_sine = amp_decay_s;
	sd->balance = balance;
	sd->t = sd->t_freq1 = sd->t_freq2 = -1;
	sd->hig = sd->mid = sd->low = 0.0;
	return;
	}

void snare_drum_start(snare_drum *sd) {
	sd->t = sd->t_freq1 = sd->t_freq2 = 0;
	sd->hig = sd->mid = sd->low = 0.0;
	return;
	}

void snare_drum_stop(snare_drum *sd) {
	sd->t = sd->t_freq1 = sd->t_freq2 = -1;
	sd->hig = sd->mid = sd->low = 0.0;
	return;
	}

void snare_drum_work(snare_drum *sd, float *output) {
	int i;
	int adecay_n, adecay_s;
	float amp1, amp2, inc1, inc2;
	float wave, noise;
	float wave_amp, noise_amp;
	float low, mid, hig;
	float freq1, freq2;

	if (sd->t < 0) {
		memset(output, 0, BUFFER_SIZE * sizeof(float));
		return;
		}
	/* obtenemos el nmero de samples para las dos rampas de cada */
	adecay_n = GET_SAMPLES_MS(sd->amp_decay_time_noise);
	adecay_s = GET_SAMPLES_MS(sd->amp_decay_time_sine);
	/* inicializamos el filtro */
	low = sd->low;
	mid = sd->mid;
	hig = sd->hig;
	for (i = 0; i < BUFFER_SIZE; i++) {
		/* amplitud del ruido */
		if (adecay_n == 0)
			amp1 = 0.0;
		 else
			amp1 = 1.0 - ((float) sd->t / adecay_n);
		/* amplitud de los dos senos */
		if (adecay_s == 0)
			amp2 = 0.0;
		 else
			amp2 = 1.0 - ((float) sd->t / adecay_s);
		if ((amp1 < 0.0) && (amp2 < 0.0)) {
			sd->t = -1;
			break;
			}
		amp1 = (amp1 < 0) ? 0 : amp1;
		amp2 = (amp2 < 0) ? 0 : amp2;
		/* calculamos la frecuencia de los dos senos */
		freq1 = sd->final_freq;
		freq1 += (sd->init_freq - sd->final_freq) *
		        (1.0 / exp(sd->exp_scale * ((float)sd->t / SAMPLE_RATE)));
		/* el segundo seno est desafinado con respecto al primero */
		freq2 = freq1 * sd->detune;
		/* calculamos los incrementos para la tabla de ondas */
		inc1 = ((float) freq1) / LOWEST_MUSIC_FREQ;
		inc2 = ((float) freq2) / LOWEST_MUSIC_FREQ;
		/* obtenemos las seales senoidales y de ruido */
		wave = sin_wave[(int) rint(sd->t_freq1)];
		if (sd->detune != 1.0)
			wave = (wave / 2.0) + (sin_wave[(int) rint(sd->t_freq2)] / 2.0);
		noise = (2.0 * (((float) rand()) / RAND_MAX)) - 1.0;
		/* equilibrio */
		noise_amp = wave_amp = 1.0;
		if (sd->balance > 0)
			wave_amp = 1.0 - sd->balance;
		else if (sd->balance < 0)
			noise_amp = 1.0 + sd->balance;
		/* obtenemos las seales senoidales */
		output[i] = wave_amp * wave * amp2;
		/* filtro para la seal de ruido nicamente */
		hig = (noise_amp * noise * amp1) - (mid * sd->filter_resonance) - low;
		mid += hig * sd->filter_cutoff;
		low += mid * sd->filter_cutoff;
		/* sumamos a las seales senoidales el ruido filtrado */
		switch (sd->filter_type) {
			case FILTERTYPE_LOWPASS  : output[i] += low; break;
			case FILTERTYPE_HIGHPASS : output[i] += hig; break;
			case FILTERTYPE_BANDPASS : output[i] += mid; break;
			case FILTERTYPE_NOTCH    : output[i] += low + hig; break;
			}
		/* actualizamos los ndices de la tabla de ondas */
		if (sd->t_freq1 >= WAVETABLE_SIZE)
			sd->t_freq1 -= WAVETABLE_SIZE;
		sd->t_freq1 += inc1;
		if (sd->t_freq2 >= WAVETABLE_SIZE)
			sd->t_freq2 -= WAVETABLE_SIZE;
		sd->t_freq2 += inc2;
		/* a por la siguiente muestra */
		sd->t++;
		}
	/* almacenamos el estado del filtro en la estructura */
	sd->hig = hig;
	sd->mid = mid;
	sd->low = low;
	/* silenciamos las muestras restantes (si las hubiere) */
	for ( ; i < BUFFER_SIZE; i++)
		output[i] = 0;
	return;
	}

/******************************************************************************
                                  BASS DRUM
******************************************************************************/
void bass_drum_init(bass_drum *bd, int ifreq, int ffreq, float exp_scale,
                    float distort, int amp_attack, int amp_decay,
                    float tick_level, int tick_length) {
	bd->init_freq = ifreq;
	bd->final_freq = ffreq;
	bd->exp_scale = exp_scale;
	bd->distort = distort;
	bd->amp_attack_time = amp_attack;
	bd->amp_decay_time = amp_decay;
	bd->tick_decay_time = tick_length;
	bd->tick_level = tick_level;
	bd->t = bd->t_freq = -1;
	return;
	}

void bass_drum_start(bass_drum *bd) {
	if (bd->t < 0)
		bd->t_freq = 0;
	bd->t = 0;
	return;
	}

void bass_drum_stop(bass_drum *bd) {
	bd->t = bd->t_freq = -1;
	return;
	}

void bass_drum_work(bass_drum *bd, float *output) {
	int i;
	int aattack, adecay, ndecay;
	float amp, n_amp;
	float freq;
	float inc;
	float sig;

	if (bd->t < 0) {
		memset(output, 0, BUFFER_SIZE * sizeof(float));
		return;
		}
	/* obtenemos las muestras de los diferentes tiempos */
	aattack = GET_SAMPLES_MS(bd->amp_attack_time);
	adecay = GET_SAMPLES_MS(bd->amp_decay_time);
	ndecay = GET_SAMPLES_MS(bd->tick_decay_time);
	for (i = 0; i < BUFFER_SIZE; i++) {
		/* clculo de la amplitud de la onda senoidal */
		amp = 0.0;
		if (bd->t < aattack)
			amp = (float)bd->t / aattack;
		else if (bd->t >= aattack)
			amp = 1.0 - ((float)(bd->t - aattack) / adecay);
		if (amp < 0.0) {
			bd->t = -1;
			break;
			}
		/* clculo de la amplitud del ruido (tick) */
		n_amp = 1.0 - ((float) bd->t / ndecay);
		n_amp = (n_amp < 0.0) ? 0.0 : n_amp;
		/* clculo de la frecuencia de la onda senoidal */
		freq = bd->final_freq;
		freq += (bd->init_freq - bd->final_freq) *
		        (1 / exp(bd->exp_scale * ((float)bd->t / SAMPLE_RATE)));
		inc = freq / LOWEST_MUSIC_FREQ;
		/* clculo de la seal final */
		sig = sin_wave[(int) rint(bd->t_freq)];
		sig = pow((sig + 1.0) / 2.0, bd->distort) * 2.0 - 1.0;
		output[i] = amp * sig;
		output[i] += n_amp * bd->tick_level * ((2.0 * rand() / RAND_MAX) - 1.0);
		bd->t_freq += inc;
		if (bd->t_freq >= WAVETABLE_SIZE)
			bd->t_freq -= WAVETABLE_SIZE;
		bd->t++;
		}
	for ( ; i < BUFFER_SIZE; i++)
		output[i] = 0;
	return;
	}

/******************************************************************************
                                    CLAP
******************************************************************************/
void clap_init(clap *cp, int amp_attack, int amp_decay, int delay, int triggs,
               float variation, int filter, float cutoff, float reso) {
	cp->amp_attack_time = amp_attack;
	cp->amp_decay_time = amp_decay;
	cp->num_triggs = triggs;
	cp->delay_time = delay;
	cp->variation = variation;
	cp->filter_type = filter;
	cp->filter_cutoff = cutoff;
	cp->filter_resonance = reso;
	cp->hig = cp->mid = cp->low = 0.0;
	cp->t = -1;
	cp->trig = 0;
	cp->curr_delay = GET_SAMPLES_MS(delay);
	cp->curr_clap_volume = 1.0 / triggs;
	return;
	}

void clap_start(clap *cp) {
	cp->t = 0;
	cp->trig = 0;
	return;
	}

void clap_stop(clap *cp) {
	cp->t = -1;
	cp->trig = 0;
	return;
	}

void clap_work(clap *cp, float *output) {
	int i;
	float amp, noise, max_amp;
	float low, mid, hig;
	int aattack, adecay;
	float amp_variation;

	if (cp->t < 0) {
		memset(output, 0, BUFFER_SIZE * sizeof(float));
		return;
		}
	aattack = GET_SAMPLES_MS(cp->amp_attack_time);
	adecay = GET_SAMPLES_MS(cp->amp_decay_time);
	low = cp->low;
	mid = cp->mid;
	hig = cp->hig;
	for (i = 0; i < BUFFER_SIZE; i++) {
		max_amp = (float)(cp->trig + 1) / (cp->num_triggs + 1);
		if (cp->t <= aattack)
			amp = cp->t * (max_amp / aattack);
		else if (cp->t <= (aattack + adecay))
			amp = max_amp - ((cp->t - aattack) * (max_amp / adecay));
		else
			amp = 0.0;
		noise = 2.0 * ((float)rand() / RAND_MAX) - 1.0;
		output[i] = noise * amp;
		hig = output[i] - (mid * cp->filter_resonance) - low;
		mid += hig * cp->filter_cutoff;
		low += mid * cp->filter_cutoff;
		switch (cp->filter_type) {
			case FILTERTYPE_LOWPASS  : output[i] = low; break;
			case FILTERTYPE_BANDPASS : output[i] = mid; break;
			case FILTERTYPE_HIGHPASS : output[i] = hig; break;
			case FILTERTYPE_NOTCH    : output[i] = low + hig; break;
			default:
				output[i] = mid;
			}
		if ((cp->t >= cp->curr_delay) && (amp > 0) &&
		                                 (cp->trig == (cp->num_triggs - 1)))
			cp->t++;
		else if (cp->t >= cp->curr_delay) {
			if (cp->trig == (cp->num_triggs - 1)) {
				cp->t = -1;
				cp->trig = 0;
				break;
				}
			cp->t = 0;
			cp->trig++;
			cp->curr_delay = GET_SAMPLES_MS(cp->delay_time);
			noise = 2.0 * ((float)rand() / RAND_MAX) - 1.0;
			cp->curr_delay += (int)rint(noise * cp->variation);
			}
		 else
			cp->t++;
		}
	for ( ; i < BUFFER_SIZE; i++)
		output[i] = 0.0;
	cp->hig = hig;
	cp->mid = mid;
	cp->low = low;
	return;
	}

/******************************************************************************
	Scale buffer array for scale constant.
*/
void buffer_const_scale(float *buffer, float scale) {
	int i;

	for (i = 0; i < BUFFER_SIZE; i++)
		buffer[i] *= scale;
	return;
	}

/******************************************************************************
	Add source1 and source2 buffers into dest buffer.
*/
void buffer_add(float *dest, float *source1, float *source2) {
	int i;

	for (i = 0; i < BUFFER_SIZE; i++)
		dest[i] = source1[i] + source2[i];
	return;
	}

/******************************************************************************
	Plays source buffers throw /dev/dsp.
*/
void buffer_play(int fd, float *left, float *right, float gain) {
	int i;
	int trash;
	short int temp_buffer[BUFFER_SIZE * 2];

	for (i = 0; i < BUFFER_SIZE; i++) {
		left[i] *= gain;
		if (left[i] > 1.0)
			left[i] = 1.0;
		else if (left[i] < -1.0)
			left[i] = -1.0;
		temp_buffer[2 * i] = (short) rint(left[i] * 32767);
		right[i] *= gain;
		if (right[i] > 1.0)
			right[i] = 1.0;
		else if (right[i] < -1.0)
			right[i] = -1.0;
		temp_buffer[(2 * i) + 1] = (short) rint(right[i] * 32767);
		}
	write(fd, temp_buffer, BUFFER_SIZE * sizeof(short int) * 2);
	return;
	}

/******************************************************************************
	Open /dev/dsp for listen.
*/
int audio_open(void) {
	int rate = SAMPLE_RATE;
	int format = AFMT_S16_LE;
	int stereo = 1;
	int fd;

	if ((fd = open("/dev/dsp", O_WRONLY, 0)) == -1) {
		perror("/dev/dsp");
		exit(-1);
		}
	ioctl(fd, SNDCTL_DSP_SPEED, &rate);
	ioctl(fd, SNDCTL_DSP_SETFMT, &format);
	ioctl(fd, SNDCTL_DSP_STEREO, &stereo);
	return fd;
	}


/*****************************************************************************/
int main(int argc, char *argv[]) {
	bass_drum bd;
	snare_drum sd;
	hihat hh;
	hihat cy;
	clap cp;
	int fd;
	float bd_output[BUFFER_SIZE];
	float sd_output[BUFFER_SIZE];
	float hh_output[BUFFER_SIZE];
	float cy_output[BUFFER_SIZE];
	float cp_output[BUFFER_SIZE];
	float output[BUFFER_SIZE];
	int i, seq_index;

	/*  open /dev/dsp. */
	if ((argc > 1) && (strcmp(argv[1], "-") == 0))
		fd = 1;
	 else
		fd = audio_open();

	/*  initializing wavetables. */
	init_wavetable();

	bass_drum_init(&bd, 300, 50, 60.0, 1.7, 100, 1500, 0.05, 30);
	memset(bass_drum_sequence, 0, SEQUENCE_SIZE * sizeof(row));
	bass_drum_sequence[0].trig = 1;
	bass_drum_sequence[0].volume = 1;
	bass_drum_sequence[4].trig = 1;
	bass_drum_sequence[4].volume = 1;
	bass_drum_sequence[8].trig = 1;
	bass_drum_sequence[8].volume = 1;
	bass_drum_sequence[12].trig = 1;
	bass_drum_sequence[12].volume = 1;

	snare_drum_init(&sd, 200, 100, 40.0, 1.1, FILTERTYPE_BANDPASS, 0.8, 0.4,
	                 1000, 350, -0.5);
	memset(snare_drum_sequence, 0, SEQUENCE_SIZE * sizeof(row));
	snare_drum_sequence[4].trig = 1;
	snare_drum_sequence[4].volume = 1;
	snare_drum_sequence[12].trig = 1;
	snare_drum_sequence[12].volume = 1;
	snare_drum_sequence[15].trig = 1;
	snare_drum_sequence[15].volume = 1;

	hihat_init(&hh, 1000, 2000, 1100, 100,
	                 300,  100,  600,   10,
	                 0.9,  0.3,  0.5,   0.4,
	           FILTERTYPE_HIGHPASS, 0.9, 0.2, 500, 0, 100);
	memset(hihat_sequence, 0, SEQUENCE_SIZE * sizeof(row));
	hihat_sequence[0].trig = 1;
	hihat_sequence[0].volume = 0.01;
	hihat_sequence[1].trig = 1;
	hihat_sequence[1].volume = 0.01;
	hihat_sequence[2].trig = 1;
	hihat_sequence[2].volume = 0.01;
	hihat_sequence[3].trig = 1;
	hihat_sequence[3].volume = 0.01;
	hihat_sequence[5].trig = 1;
	hihat_sequence[5].volume = 0.01;
	hihat_sequence[6].trig = 1;
	hihat_sequence[6].volume = 0.01;
	hihat_sequence[7].trig = 1;
	hihat_sequence[7].volume = 0.005;
	hihat_sequence[8].trig = 1;
	hihat_sequence[8].volume = 0.03;
	hihat_sequence[9].trig = 1;
	hihat_sequence[9].volume = 0.01;
	hihat_sequence[10].trig = 1;
	hihat_sequence[10].volume = 0.001;
	hihat_sequence[11].trig = 1;
	hihat_sequence[11].volume = 0.015;
	hihat_sequence[13].trig = 1;
	hihat_sequence[13].volume = 0.05;
	hihat_sequence[15].trig = 1;
	hihat_sequence[15].volume = 0.01;

	hihat_init(&cy, 5000, 6000, 8100, 1000,
	                 50, 60, 60,   90,
	                 0.9,  0.5,  0.3,   0.1,
	           FILTERTYPE_HIGHPASS, 0.8, 0.2, 1700, -0.5, -1);
	memset(cymbal_sequence, 0, SEQUENCE_SIZE * sizeof(row));
	cymbal_sequence[2].trig = 1;
	cymbal_sequence[2].volume = 0.05;
	cymbal_sequence[6].trig = 1;
	cymbal_sequence[6].volume = 0.05;
	cymbal_sequence[9].trig = 1;
	cymbal_sequence[9].volume = 0.01;
	cymbal_sequence[10].trig = 1;
	cymbal_sequence[10].volume = 0.05;
	cymbal_sequence[14].trig = 1 ;
	cymbal_sequence[14].volume = 0.05;

	clap_init(&cp, 10, 360, 90, 4, 0, FILTERTYPE_HIGHPASS, 0.17, 0.27);
	memset(clap_sequence, 0, SEQUENCE_SIZE * sizeof(row));
	clap_sequence[4].trig = 1;
	clap_sequence[4].volume = 0.4;
	clap_sequence[12].trig = 1;
	clap_sequence[12].volume = 0.4;

	seq_index = 0;
	i = 0;
	while (1) {
		memset(output, 0, BUFFER_SIZE * sizeof(float));
		if (i >= SAMPLES_PER_ROW) {
			i = 0;
			if (bass_drum_sequence[seq_index].trig > 0)
				bass_drum_sequence[seq_index].trig = 1;
			if (snare_drum_sequence[seq_index].trig > 0)
				snare_drum_sequence[seq_index].trig = 1;
			if (hihat_sequence[seq_index].trig > 0)
				hihat_sequence[seq_index].trig = 1;
			if (cymbal_sequence[seq_index].trig > 0)
				cymbal_sequence[seq_index].trig = 1;
			if (clap_sequence[seq_index].trig > 0)
				clap_sequence[seq_index].trig = 1;
			seq_index = (seq_index + 1) % SEQUENCE_SIZE;
			}
		if (bass_drum_sequence[seq_index].trig == 1) {
			bass_drum_start(&bd);
			bass_drum_sequence[seq_index].trig = 2;
			bass_drum_volume = bass_drum_sequence[seq_index].volume;
			}
		if (snare_drum_sequence[seq_index].trig == 1) {
			snare_drum_start(&sd);
			snare_drum_sequence[seq_index].trig = 2;
			snare_drum_volume = snare_drum_sequence[seq_index].volume;
			}
		if (hihat_sequence[seq_index].trig == 1) {
			hihat_start(&hh);
			hihat_sequence[seq_index].trig = 2;
			hihat_volume = hihat_sequence[seq_index].volume;
			}
		if (cymbal_sequence[seq_index].trig == 1) {
			hihat_start(&cy);
			cymbal_sequence[seq_index].trig = 2;
			cymbal_volume = cymbal_sequence[seq_index].volume;
			}
		if (clap_sequence[seq_index].trig == 1) {
			clap_start(&cp);
			clap_sequence[seq_index].trig = 2;
			clap_volume = clap_sequence[seq_index].volume;
			}
		bass_drum_work(&bd, bd_output);
		buffer_const_scale(bd_output, bass_drum_volume);
		snare_drum_work(&sd, sd_output);
		buffer_const_scale(sd_output, snare_drum_volume);
		hihat_work(&hh, hh_output);
		buffer_const_scale(hh_output, hihat_volume);
		hihat_work(&cy, cy_output);
		buffer_const_scale(cy_output, cymbal_volume);
		clap_work(&cp, cp_output);
		buffer_const_scale(cp_output, clap_volume);
		buffer_add(output, bd_output, sd_output);
		buffer_add(output, output, hh_output);
		buffer_add(output, output, cy_output);
		buffer_add(output, output, cp_output);
		buffer_play(fd, output, output, 1.0);
		i += BUFFER_SIZE;
		}

	/*  close /dev/dsp. */
	close(fd);
	return 0;
	}
