/*
 * (c) Avelino Herrera Morales
 *
 * Ejemplo de sntesis mediante distorsin de fase (PD).
 *
	This software is free software; you can redistribute it and/or modify
	it under the terms of the GNU  General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
																																																																																																							 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU  General Public License for more details.
																																																																																																							 
	You should have received a copy of the GNU  General Public
	License along with this software; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include "audio.h"


#define  F_SENIAL   220    /* un LA en la 3 octava del piano */
#define  F_MUEST    44100  /* frecuencia de muetreo: 44.1 KHz */
#define  TIEMPO     2      /* segundos de reproduccin */

#define  MUESTRAS_TOTALES  (TIEMPO * F_MUEST)
/* incremento en la variable fase, dicha variable va de 0 a 1 en cada perodo */
#define  INC_FASE          ((float)F_SENIAL / F_MUEST)

signed short int salida[MUESTRAS_TOTALES];

/* una distorsin sencillita */
float distorsion1(float fase) {
	return (float)pow(fase, 3.0);
	}

/* otra distorsin, esta vez definida como una funcin a trozos */
float distorsion2(float fase) {
	if (fase < 0.5)
		return (float)pow(fase, 2.0);
	else if (fase < 0.75)
		return (fase - 0.25);
	 else
		return 0.80;
	}

int main(void) {
	int i;
	int fd;
	float v;
	float fase;

	fd = audio_open(0);
	/* empezamos con fase 0 */
	fase = 0;
	for (i = 0; i < MUESTRAS_TOTALES; i++) {
		/* calculamos la seal a partir de la fase aplicando un algoritmo de
		   distorsin que devuelva un valor entre 0 y 1 (como la variable fase,
		   que toma valores entre 0 y 1)
		   si no queremos que haya distorsin de fase simplemente sustituimos
		   la lnea de cdigo por v = cos(2 * M_PI * fase) */
		v = cos(2 * M_PI * distorsion1(fase));
		salida[i] = (signed short int) rint(v * 32767);
		/* incrementamos la fase */
		fase += INC_FASE;
		/* mantenemos la variable fase dentro del intervalo [0, 1] */
		if (fase > 1)
			fase -= 1;
		}
	write(fd, salida, MUESTRAS_TOTALES * sizeof(signed short int));
	return 0;
	}
