/*
 * (c) Avelino Herrera Morales
 *
 * Ejemplo de mezcla de seales con un ligero desfase en frecuencia.
 *
	This software is free software; you can redistribute it and/or modify
	it under the terms of the GNU  General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
																																																																																																							 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU  General Public License for more details.
																																																																																																							 
	You should have received a copy of the GNU  General Public
	License along with this software; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include "audio.h"


#define  F_SENIAL   220    /* un LA en la 3 octava del piano */
#define  F_MUEST    44100  /* frecuencia de muetreo: 44.1 KHz */
#define  MULT       1.008  /* factor de frecuencia para el segundo oscilador */
#define  TIEMPO     2      /* segundos de reproduccin */

#define  MUESTRAS_TOTALES  (TIEMPO * F_MUEST)
/* incrementos de la fase para cada oscilador */
#define  INC_FASE1         ((float)F_SENIAL / F_MUEST)
#define  INC_FASE2         ((float)(F_SENIAL * MULT) / F_MUEST)

#define  SYNC  1 /* a 0 para no sincronizar, a 1 para sincronizar */

signed short int salida[MUESTRAS_TOTALES];

int main(void) {
	int i;
	int fd;
	float v1, v2, v;
	float fase1, fase2;

	fd = audio_open(0);
	/* inicializamos las fases a 0 */
	fase1 = fase2 = 0;
	for (i = 0; i < MUESTRAS_TOTALES; i++) {
		/* primer oscilador */
		v1 = cos(2 * M_PI * fase1);
		v1 = (v1 > 0) ? 1.0 : -1.0;
		/* segundo oscilador */
		v2 = cos(2 * M_PI * fase2);
		v2 = (v2 > 0) ? 1.0 : -1.0;
		/* sumamos */
		v = (v1 + v2) / 2.0;
		salida[i] = (signed short int) rint(v * 32767);
		/* incrementamos la fases y sincronizados, si es preciso */
		fase2 += INC_FASE2;
		if (fase2 > 1)
			fase2 -= 1;
		fase1 += INC_FASE1;
		if (fase1 > 1) {
			fase1 -= 1;
			if (SYNC)
				fase2 = 0;
			}
		}
	write(fd, salida, MUESTRAS_TOTALES * sizeof(signed short int));
	return 0;
	}
