/*
 * (c) Avelino Herrera Morales
 *
 * Ejemplo de sntesis vectorial con tres componentes.
 *
	This software is free software; you can redistribute it and/or modify
	it under the terms of the GNU  General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
																																																																																																							 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU  General Public License for more details.
																																																																																																							 
	You should have received a copy of the GNU  General Public
	License along with this software; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include "audio.h"

#define  F_SENIAL       220     /* LA en la 3 octava del piano */
#define  F_MUEST        44100   /* frecuencia de muestreo */
#define  TIEMPO         2       /* duracin total en segundos */
#define  TIEMPO_ATAQUE  0.5     /* tiempo de ataque en segundos */
#define  TIEMPO_CAIDA   1.0     /* tiempo de caida en segundos */

#define  MUESTRAS_TOTALES  (TIEMPO * F_MUEST)
#define  MUESTRAS_ATAQUE   ((int) rint(TIEMPO_ATAQUE * F_MUEST))
#define  MUESTRAS_CAIDA    ((int) rint(TIEMPO_CAIDA * F_MUEST))

/* variables para el control del oscilador de diente de sierra */
int ds_t = 0;
int ds_max_t;
float ds_ramp;
/* control de la envolvente */
int env_t = 0;

/* arrays donde meteremos las diferentes seales */
float salida_diente_sierra[MUESTRAS_TOTALES];
float salida_cuadrada1[MUESTRAS_TOTALES];
float salida_cuadrada2[MUESTRAS_TOTALES];
float salida_envolvente[MUESTRAS_TOTALES];
float mezcla[MUESTRAS_TOTALES];
/* array de salida para enviar directamente al dispositivo */
signed short int salida[MUESTRAS_TOTALES];

/******************************************************************************
	Inicializa las variables para generar la onda de diente de sierra en tiempo
	real mediante el clculo de la rampa.
*/
void diente_sierra_inicia(float frec) {
	ds_t = 0;
	/* ds_max_t es el nmero de muestras por perodo de la seal */
	ds_max_t = (int) rint(F_MUEST / frec);
	ds_max_t--;
	/* pendiente del diente de sierra */
	ds_ramp = 2.0 / ds_max_t;
	return;
	}

/******************************************************************************
	Genera n muestras en el array out de la seal de diente de sierra.
*/
void diente_sierra_trabaja(float *out, int n) {
	int i;

	for (i = 0; i < n; i++) {
		/* generamos la salida multiplicando la t del oscilador por la rampa */
		out[i] = -1.0 + (ds_t * ds_ramp);
		ds_t++;
		if (ds_t > ds_max_t)
			ds_t = 0;
		}
	return;
	}

/******************************************************************************
	Genera, a partir de una seal de entrada, una onda cuadrada con anchura
	ajustable a pwm.
*/
void cuadrada_genera(float *out, float *ds_in, int n, float pwm) {
	int i;

	for (i = 0; i < n; i++)
		/* si la seal de entrada supera pwm emitimos un +1, si no un -1 */
		out[i] = (ds_in[i] > pwm) ? 1.0 : -1.0;
	return;
	}

/******************************************************************************
	Genera n muestras sobre el array out de una envolvente de tipo AD (valores
	de salida entre 0 y 1).
*/
void envolvente_genera(float *out, int n) {
	int i;

	for (i = 0; i < n; i++) {
		/* perodo de ataque */
		if (env_t <= MUESTRAS_ATAQUE)
			out[i] = (float)env_t / MUESTRAS_ATAQUE;
		/* perodo de caida */
		else if (env_t <= (MUESTRAS_ATAQUE + MUESTRAS_CAIDA)) {
			out[i] = ((float)(env_t - MUESTRAS_ATAQUE) / MUESTRAS_CAIDA);
			out[i] = 1.0 - out[i];
			}
		/* fin de la envolvente */
		else
			out[i] = 0;
		env_t++;
		}
	return;
	}

/*****************************************************************************/
int main(void) {
	int i;
	int fd;
	float nivel1, nivel2, nivel3;

	fd = audio_open(0);
	/* generamos las diferentes seales sobre los arrays */
	diente_sierra_inicia(F_SENIAL);
	diente_sierra_trabaja(salida_diente_sierra, MUESTRAS_TOTALES);
	cuadrada_genera(salida_cuadrada1, salida_diente_sierra, MUESTRAS_TOTALES,
                    0);
	cuadrada_genera(salida_cuadrada2, salida_diente_sierra, MUESTRAS_TOTALES,
                    0.8);
	envolvente_genera(salida_envolvente, MUESTRAS_TOTALES);
	/* hacemos la secuenciacin en funcin de la envolvente */
	for (i = 0; i < MUESTRAS_TOTALES; i++) {
		/* expandimos la envolvente de [0, 1] a [-1, +1] */
		salida_envolvente[i] = (2.0 * salida_envolvente[i]) - 1.0;
		/* calculamos los niveles para cada onda */
		if (salida_envolvente[i] < 0) {
			nivel3 = 0.0;
			nivel1 = -salida_envolvente[i];
			nivel2 = 1.0 - nivel1;
			}
		else {
			nivel1 = 0.0;
			nivel3 = salida_envolvente[i];
			nivel2 = 1.0 - nivel3;
			}
		/* calculamos la mezcla final de seales */
		mezcla[i] = (nivel1 * salida_diente_sierra[i]) +
		            (nivel2 * salida_cuadrada1[i]) +
		            (nivel3 * salida_cuadrada2[i]);
		/* convertimos la mezcla final para emitirla por el dispositivo */
		if (mezcla[i] > 1.0)
			mezcla[i] = 1.0;
		else if (mezcla[i] < -1.0)
			mezcla[i] = -1.0;
		salida[i] = (int) rint(mezcla[i] * 32767);
		}
	write(fd, salida, sizeof(signed short int) * MUESTRAS_TOTALES);
	close(fd);
	return 0;
	}
