/*
 * (c) Avelino Herrera Morales
 *
 * Ejemplo de sntesis mediante crossfade.
 * Modulacin del balance de seales a frecuencia audible.
 *
	This software is free software; you can redistribute it and/or modify
	it under the terms of the GNU  General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
																																																																																																							 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU  General Public License for more details.
																																																																																																							 
	You should have received a copy of the GNU  General Public
	License along with this software; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA.
*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include "audio.h"

#define  F_SENIAL       220     /* LA en la 3 octava del piano */
#define  F_CROSSFADE    111     /* frecuencia del crossfade */
#define  F_MUEST        44100   /* frecuencia de muestreo */
#define  TIEMPO         2       /* duracin total en segundos */
#define  TIEMPO_ATAQUE  0.5     /* tiempo de ataque en segundos */
#define  TIEMPO_CAIDA   1.0     /* tiempo de caida en segundos */

#define  MUESTRAS_TOTALES  (TIEMPO * F_MUEST)

/* variables para el control del oscilador de diente de sierra */
int ds1_t = 0;
int ds1_max_t;
float ds1_ramp;
int ds2_t = 0;
int ds2_max_t;
float ds2_ramp;

/* arrays donde meteremos las diferentes seales */
float salida_diente_sierra[MUESTRAS_TOTALES];
float salida_cuadrada[MUESTRAS_TOTALES];
float salida_crossfade[MUESTRAS_TOTALES];
float mezcla[MUESTRAS_TOTALES];
/* array de salida para enviar directamente al dispositivo */
signed short int salida[MUESTRAS_TOTALES];


/******************************************************************************
	Inicializa las variables para generar la onda de diente de sierra en tiempo
	real mediante el clculo de la rampa.
*/
void diente_sierra1_inicia(float frec) {
	ds1_t = 0;
	/* ds_max_t es el nmero de muestras por perodo de la seal */
	ds1_max_t = (int) rint(F_MUEST / frec);
	ds1_max_t--;
	/* pendiente del diente de sierra */
	ds1_ramp = 2.0 / ds1_max_t;
	return;
	}

void diente_sierra2_inicia(float frec) {
	ds2_t = 0;
	/* ds_max_t es el nmero de muestras por perodo de la seal */
	ds2_max_t = (int) rint(F_MUEST / frec);
	ds2_max_t--;
	/* pendiente del diente de sierra */
	ds2_ramp = 2.0 / ds2_max_t;
	return;
	}

/******************************************************************************
	Genera n muestras en el array out de la seal de diente de sierra.
*/
void diente_sierra1_trabaja(float *out, int n) {
	int i;

	for (i = 0; i < n; i++) {
		/* generamos la salida multiplicando la t del oscilador por la rampa */
		out[i] = -1.0 + (ds1_t * ds1_ramp);
		ds1_t++;
		if (ds1_t > ds1_max_t)
			ds1_t = 0;
		}
	return;
	}

void diente_sierra2_trabaja(float *out, int n) {
	int i;

	for (i = 0; i < n; i++) {
		/* generamos la salida multiplicando la t del oscilador por la rampa */
		out[i] = -1.0 + (ds2_t * ds2_ramp);
		ds2_t++;
		if (ds2_t > ds2_max_t)
			ds2_t = 0;
		}
	return;
	}

/******************************************************************************
	Genera, a partir de una seal de entrada, una onda cuadrada con anchura
	ajustable a pwm.
*/
void cuadrada_genera(float *out, float *ds_in, int n, float pwm) {
	int i;

	for (i = 0; i < n; i++)
		/* si la seal de entrada supera pwm emitimos un +1, si no un -1 */
		out[i] = (ds_in[i] > pwm) ? 1.0 : -1.0;
	return;
	}

/*****************************************************************************/
int main(void) {
	int i;
	int fd;
	float nivel1, nivel2;

	fd = audio_open(0);
	/* generamos las diferentes seales sobre los arrays */
	diente_sierra1_inicia(F_SENIAL);
	diente_sierra1_trabaja(salida_diente_sierra, MUESTRAS_TOTALES);
	cuadrada_genera(salida_cuadrada, salida_diente_sierra, MUESTRAS_TOTALES,
                    0.7);
	diente_sierra2_inicia(F_CROSSFADE);
	diente_sierra2_trabaja(salida_crossfade, MUESTRAS_TOTALES);
	/* hacemos el crossfade */
	for (i = 0; i < MUESTRAS_TOTALES; i++) {
		/* comprimimos el rango del crossfade de [-1, +1] a [0, 1] */
		salida_crossfade[i] = (salida_crossfade[i] + 1.0) / 2.0;
		/* calculamos los niveles para cada onda */
		nivel1 = 1.0 - salida_crossfade[i];
		nivel2 = salida_crossfade[i];
		/* calculamos la mezcla final de seales */
		mezcla[i] = (nivel1 * salida_diente_sierra[i]) +
		            (nivel2 * salida_cuadrada[i]);
		/* convertimos la mezcla final para emitirla por el dispositivo */
		if (mezcla[i] > 1.0)
			mezcla[i] = 1.0;
		else if (mezcla[i] < -1.0)
			mezcla[i] = -1.0;
		salida[i] = (int) rint(mezcla[i] * 32767);
		}
	write(fd, salida, sizeof(signed short int) * MUESTRAS_TOTALES);
	close(fd);
	return 0;
	}
